// klgaussian.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Apply a gaussian to a kernel.

#include "kernel/lib/kernel.h"
#include "kernel/lib/composition-kernel.h"
#include "kernel/lib/rational-kernel.h"

DEFINE_double(gamma, 1.0F, "gamma of sigmoid");

using namespace kernel;

int main(int argc, char** argv) {
  string usage = "Compose kernel with gaussian: exp(-gamma * (k[x,x] + k[y,y] - 2 k[x,y])).\n\n  Usage: ";
  usage += argv[0];
  usage += " in.kar > out.kar\n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc > 3) {
    ShowUsage();
    return 1;
  }
  string in = argc > 1 ? string(argv[1]) : string();
  string out = argc > 2 ? string(argv[2]) : string();


  Kernel< Fst<LogArc> > *kernel = Kernel< Fst<LogArc> >::Read(in);
  if(!kernel)
    LOG(FATAL) << "klgaussian: failed to read" << in;

  GaussianKernel< Fst<LogArc> > k(kernel, FLAGS_gamma);
  k.Write(out);
  delete kernel;
  return 0;
}
