// blas-compat.h
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: allauzen@google.com (Cyril)
//
// \file
// Blas/Lapack forward declarations, type definitions.

#ifndef KERNEL_LIB_BLAS_COMPAT_H__
#define KERNEL_LIB_BLAS_COMPAT_H__

#include <complex>

namespace lapack {

typedef int32 integer;
typedef int32 logical;
typedef float real;
typedef double doublereal;

}

extern "C" {

  void dgesv_(lapack::integer*, lapack::integer*, lapack::doublereal*,
              lapack::integer*, lapack::integer*, lapack::doublereal*, 
              lapack::integer*, lapack::integer*);

  void sgesv_(lapack::integer*, lapack::integer*, lapack::real*,
              lapack::integer*, lapack::integer*, lapack::real*, 
              lapack::integer*, lapack::integer*);

  void dposv_(char*, lapack::integer*, lapack::integer*, lapack::doublereal*,
              lapack::integer*, lapack::doublereal*, lapack::integer*,
              lapack::integer*);

  void sposv_(char*, lapack::integer*, lapack::integer*, lapack::real*,
              lapack::integer*, lapack::real*, lapack::integer*,
              lapack::integer*);

  void dsyev_(char*, char*, lapack::integer*, lapack::doublereal*,
              lapack::integer*, lapack::doublereal*, lapack::doublereal*,
              lapack::integer*, lapack::integer*);

  void ssyev_(char*, char*, lapack::integer*, lapack::real*, lapack::integer*,
              lapack::real*, lapack::real*, lapack::integer*, lapack::integer*);

  void dpotri_(char*, lapack::integer*, lapack::doublereal*, lapack::integer*,
               lapack::integer*);

  void spotri_(char*, lapack::integer*, lapack::real*, lapack::integer*,
               lapack::integer*);

  void dpotrf_(char*, lapack::integer*, lapack::doublereal*, lapack::integer*,
               lapack::integer*);

  void spotrf_(char*, lapack::integer*, lapack::real*, lapack::integer*,
               lapack::integer*);

  void dgetri_(lapack::integer*, lapack::doublereal*, lapack::integer*,
               lapack::integer*, lapack::doublereal*, lapack::integer*,
               lapack::integer*);

  void sgetri_(lapack::integer*, lapack::real*, lapack::integer*,
               lapack::integer*, lapack::real*, lapack::integer*,
               lapack::integer*);

  void dgetrf_(lapack::integer*, lapack::integer*, lapack::doublereal*,
               lapack::integer*, lapack::integer*, lapack::integer*);

  void sgetrf_(lapack::integer*, lapack::integer*, lapack::real*,
               lapack::integer*, lapack::integer*, lapack::integer*);

  void dgemm_(char*, char*, lapack::integer*, lapack::integer*,
              lapack::integer*, lapack::doublereal*, lapack::doublereal*,
              lapack::integer*, lapack::doublereal*, lapack::integer*,
              lapack::doublereal*, lapack::doublereal*, lapack::integer*);

  void sgemm_(char*, char*, lapack::integer*, lapack::integer*,
              lapack::integer*, lapack::real*, lapack::real*, lapack::integer*,
              lapack::real*, lapack::integer*, lapack::real*, lapack::real*,
              lapack::integer*);

  void dsymm_(char*, char*, lapack::integer*, lapack::integer*,
              lapack::doublereal*, lapack::doublereal*, lapack::integer*,
              lapack::doublereal*, lapack::integer*, lapack::doublereal*,
              lapack::doublereal*, lapack::integer*);

  void ssymm_(char*, char*, lapack::integer*, lapack::integer*, lapack::real*,
              lapack::real*, lapack::integer*, lapack::real*, lapack::integer*,
              lapack::real*, lapack::real*, lapack::integer*);

  void dsyrk_(char*, char*, lapack::integer*, lapack::integer*,
              lapack::doublereal*, lapack::doublereal*, lapack::integer*,
              lapack::doublereal*, lapack::doublereal*, lapack::integer*);

  void ssyrk_(char*, char*, lapack::integer*, lapack::integer*, lapack::real*,
              lapack::real*, lapack::integer*, lapack::real*, lapack::real*,
              lapack::integer*);

  void dgemv_(char*, lapack::integer*, lapack::integer*, lapack::doublereal*,
              lapack::doublereal*, lapack::integer*, lapack::doublereal*,
              lapack::integer*, lapack::doublereal*, lapack::doublereal*,
              lapack::integer*);

  void sgemv_(char*, lapack::integer*, lapack::integer*, lapack::real*,
              lapack::real*, lapack::integer*, lapack::real*, lapack::integer*,
              lapack::real*, lapack::real*, lapack::integer*);
}

#endif  // KERNEL_LIB_BLAS_COMPAT_H__
